<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
	<xsl:output method="html" version="4.0" encoding="UTF-8" indent="yes" />
    <xsl:strip-space elements="*"/>
	
	<xsl:key name="statedifference-id" match="stateDifference" use="@differenceId"/>
    <xsl:key name="difference-id" match="difference" use="@differenceId"/>
    <xsl:key name="rootElementdifference-id" match="rootElementDifference" use="@screenId"/>
    <xsl:key name="elementDifference-id" match="elementDifference" use="@retestId"/>
    <xsl:key name="attributeDifference-id" match="attributeDifference" use="@attributeDifferenceId"/>
	<xsl:variable name="stateDifference" select="//stateDifference[generate-id()= generate-id(key('statedifference-id', @differenceId)[1])]"/>
	<xsl:variable name="difference" select="//difference[generate-id()= generate-id(key('difference-id', @differenceId)[1])]"/>
 	<xsl:variable name="rootElementDifference" select="//rootElementDifference[generate-id()= generate-id(key('rootElementdifference-id', @screenId)[1])]"/>
 	<xsl:variable name="elementDifference" select="//elementDifference[not(descendant::elementDifference)][generate-id()= generate-id(key('elementDifference-id', @retestId)[1])]"/>
	<xsl:variable name="attributeDifference" select="//attributeDifference[generate-id()= generate-id(key('attributeDifference-id', @attributeDifferenceId)[1])]"/>
	
	<xsl:variable name="totalErrors" select="count(//error[not(./@errorId = ./preceding::error/@errorId)]) + count(//targetNotFound)"></xsl:variable>
	
	<xsl:template match="reTestXmlDataContainer">
		<html>
			<head>
				<title>ReTest generated Testreport</title>
				<meta charset="utf-8" />
				<meta name="viewport"
					content="width=device-width, initial-scale=1, maximum-scale=1" />
				<link rel="stylesheet" href="de/retest/css/default.css" />
				<link rel="stylesheet" href="de/retest/css/layout.css" />
				<link rel="stylesheet" href="de/retest/css/media-queries.css" />
				<script src="de/retest/js/modernizr.js"></script>
				<script src="de/retest/js/jquery-1.12.3.min.js"></script>
				<script src="de/retest/js/retest.js"></script>
				<link rel="shortcut icon" href="de/retest/img/favicon.ico" />
				<link rel="apple-touch-icon" href="de/retest/img/apple-touch-icon.png" />
				<script type="text/javascript">
				 var diffs2Screens = {};
				 var diffs2Comps = {};
				 var diffs2Attrs = {};
				 <xsl:for-each select="$stateDifference">
				    <xsl:variable name="differenceId" select="@differenceId"/>
				 		<xsl:for-each select="child::rootElementDifference">
				 		 push(diffs2Screens, 'difference<xsl:value-of select="$differenceId"/>', 'screen<xsl:value-of select="@screenId"/>');</xsl:for-each>
					 	<xsl:for-each select="descendant::elementDifference[not(descendant::elementDifference)]">
					 	 push(diffs2Comps, 'difference<xsl:value-of select="$differenceId"/>', 'component<xsl:value-of select="@retestId"/>');</xsl:for-each>
                        <xsl:for-each select="descendant::attributeDifference[@attributeDifferenceId and not(@attributeDifferenceId = ./preceding::attributeDifference/@attributeDifferenceId)]">
                         push(diffs2Attrs, 'difference<xsl:value-of select="$differenceId"/>', 'attributeDifference<xsl:value-of select="@attributeDifferenceId"/>');</xsl:for-each>
				 </xsl:for-each>
				 var screens2Diffs = {};
				 var screens2Comps = {};
				 var screens2Attrs = {};
				 <xsl:for-each select="$stateDifference//rootElementDifference">
				 	 <xsl:variable name="screenId" select="@screenId"/>
					 <xsl:for-each select="ancestor::stateDifference">
					 push(screens2Diffs, 'screen<xsl:value-of select="$screenId"/>', 'difference<xsl:value-of select="@differenceId"/>');</xsl:for-each>
                     <xsl:for-each select="descendant::elementDifference[not(descendant::elementDifference)]">
                     push(screens2Comps, 'screen<xsl:value-of select="$screenId"/>', 'component<xsl:value-of select="@retestId"/>');</xsl:for-each>
                     <xsl:for-each select="descendant::attributeDifference[@attributeDifferenceId and not(@attributeDifferenceId = ./preceding::attributeDifference/@attributeDifferenceId)]">
                     push(screens2Attrs, 'screen<xsl:value-of select="$screenId"/>', 'attributeDifference<xsl:value-of select="@attributeDifferenceId"/>');</xsl:for-each>
				 </xsl:for-each>
                 var comps2Diffs = {};
				 var comps2Screens = {};
				 var comps2Attrs = {};
				 var attrs2Comps = {};
				 <xsl:for-each select="$stateDifference//childDifferences/elementDifference">
				 	 <xsl:variable name="retestId" select="@retestId"/>
					 <xsl:for-each select="ancestor::rootElementDifference">
					 push(comps2Screens, 'component<xsl:value-of select="$retestId"/>', 'screen<xsl:value-of select="@screenId"/>');</xsl:for-each>
					 <xsl:for-each select="ancestor::stateDifference">
					 push(comps2Diffs, 'component<xsl:value-of select="$retestId"/>', 'difference<xsl:value-of select="@differenceId"/>');</xsl:for-each>
                     <xsl:for-each select="descendant::attributeDifference[@attributeDifferenceId and not(@attributeDifferenceId = ./preceding::attributeDifference/@attributeDifferenceId)]">
                     push(comps2Attrs, 'component<xsl:value-of select="$retestId"/>', 'attributeDifference<xsl:value-of select="@attributeDifferenceId"/>');
                     push(attrs2Comps, 'attributeDifference<xsl:value-of select="@attributeDifferenceId"/>', 'component<xsl:value-of select="$retestId"/>');</xsl:for-each>
				 </xsl:for-each>
                 var attrs2Screens = {};
                 var attrs2Diffs = {};
                 <xsl:for-each select="$attributeDifference">
                     push(attrs2Screens, 'attributeDifference<xsl:value-of select="@attributeDifferenceId"/>', 'screen<xsl:value-of select="ancestor::rootElementDifference/@screenId"/>');
                     push(attrs2Diffs, 'attributeDifference<xsl:value-of select="@attributeDifferenceId"/>', 'difference<xsl:value-of select="ancestor::stateDifference/@differenceId"/>');</xsl:for-each>    
				</script>
				<style type="text/css">
				 <xsl:for-each select="$stateDifference">
				 .difference<xsl:value-of select="@differenceId"/>{background-color: #FFFE89; -webkit-transition: background-color 0.5s ease-in-out; -moz-transition: background-color 0.5s ease-in-out; -o-transition: background-color 0.5s ease-in-out; transition: background-color 0.5s ease-in-out;}</xsl:for-each>
				 <xsl:for-each select="$rootElementDifference">
				 .screen<xsl:value-of select="@screenId"/>{background-color: #FFFE89; -webkit-transition: background-color 0.5s ease-in-out; -moz-transition: background-color 0.5s ease-in-out; -o-transition: background-color 0.5s ease-in-out; transition: background-color 0.5s ease-in-out;}</xsl:for-each>
				 <xsl:for-each select="$elementDifference">
				 .component<xsl:value-of select="@retestId"/>{background-color: #FFFE89; -webkit-transition: background-color 0.5s ease-in-out; -moz-transition: background-color 0.5s ease-in-out; -o-transition: background-color 0.5s ease-in-out; transition: background-color 0.5s ease-in-out;}</xsl:for-each>
                 <xsl:for-each select="$attributeDifference">
                 .attributeDifference<xsl:value-of select="@attributeDifferenceId"/>{background-color: #FFFE89; -webkit-transition: background-color 0.5s ease-in-out; -moz-transition: background-color 0.5s ease-in-out; -o-transition: background-color 0.5s ease-in-out; transition: background-color 0.5s ease-in-out;}</xsl:for-each>
				 .errorRow {background-color: #FF9FA1;}
				</style>
			</head>
			<body class="testreport">
				<header>
					<div class="row">
						<div class="twelve columns">
							<div class="logo">
								<a href="https://retest.de/">
									<img alt="ReTest" src="de/retest/img/logo.png" />
								</a>
							</div>
							<nav id="nav-wrap">
								<a class="mobile-btn" href="#nav-wrap" title="Show navigation">Show
									navigation</a> <a class="mobile-btn" href="#" title="Hide navigation">Hide
									navigation</a>
			
								<ul id="nav" class="nav">
									<xsl:if test="$stateDifference">
									<li><a href="#differenceSummary">Difference Summary</a></li>
									</xsl:if>
									<xsl:if test="//error or //targetNotFound">
									<li><a href="#errorSummary">Error Summary</a></li>
									</xsl:if>
									<li><a href="#tests">Individual Tests</a></li>
									<li>
										<a id="closeAll" href="javascript:;">Close all
										<script type="text/javascript">
												$("#closeAll").click(function () {
													$(".height-transition-shown").slideUpTransition();
												});
										</script>
										</a>
									</li>
								</ul>
							</nav>
						</div>
					</div>
				</header>
				
				<div id="intro" style="text-align: center; margin-top: 96px;">
					<h1>ReTest ran <xsl:value-of select="count(//suite)"/> suite(s) and <xsl:value-of select="count(//test)"/> test(s) with <xsl:value-of select="count(//action)"/> replayed action(s). </h1>
					<br />
					<br />
					<a onclick="document.getElementById('intro').style.display='none';document.getElementById('content').style.display='block';" href="#">Review results &gt;</a>
					<br />
				</div>
				
				<div id="content" style="display: none;">
				<xsl:if test="$stateDifference">
				<section id="differenceSummarySection">
					<a name="differenceSummary" />
					<h1><a href="javascript:;" id="differencePerScreen">Differences per Screen</a></h1>
					<script type="text/javascript">
							$("#differencePerScreen").click(function () {            
							    if ($("#screenDiffSlider").hasClass("height-transition-hidden"))
							        $("#screenDiffSlider").slideDownTransition();
							    else
							        $("#screenDiffSlider").slideUpTransition();
							});
					</script>
					<div id="screenDiffSlider" class="height-transition height-transition-hidden"><div>
					<table class="summaryTable">
						<xsl:for-each select="$rootElementDifference">
							<xsl:sort select="@screenId" />
							<xsl:variable name="window" select="."/>
							<tr>
							<xsl:attribute name="class"> screen<xsl:value-of select="@screenId"/></xsl:attribute>
								<td>
									<input type="checkbox">
										<xsl:attribute name="name">screen<xsl:value-of select="@screenId"/>Check</xsl:attribute>
										<xsl:attribute name="id">screen<xsl:value-of select="@screenId"/>Check</xsl:attribute>
										<xsl:attribute name="onclick">markScreen('screen<xsl:value-of select="@screenId"/>');</xsl:attribute>
										<a>
										<xsl:attribute name="name">screen<xsl:value-of select="@screenId" /></xsl:attribute>
										Screen "<xsl:value-of select="@windowDescription"/>"
										</a>
									</input>
								</td>
								<td class="stateExpectedCol">
								    <xsl:choose>
								        <xsl:when test="expectedScreenshot">
									        <img class="screenShot">
	                                            <xsl:attribute name="src">screenshot/<xsl:value-of select="expectedScreenshot/persistenceId/text()" />.png</xsl:attribute>
	                                        </img>
								        </xsl:when>
								        <xsl:otherwise>
								        	<xsl:value-of select="@title" />
								        </xsl:otherwise>
								    </xsl:choose>
								</td>
								<td>
								Differences: <br />
								<xsl:for-each select="$stateDifference[.//rootElementDifference/@screenId = $window/@screenId]">
										<a>
										<xsl:attribute name="href">#difference<xsl:value-of select="@differenceId"/></xsl:attribute>
										<xsl:attribute name="id">screenDiffLink<xsl:value-of select="$window/@screenId"/></xsl:attribute>
										Difference <xsl:value-of select="@differenceId"/>
										<script type="text/javascript">
												$("#screenDiffLink<xsl:value-of select="$window/@screenId"/>").click(function () {            
												    if ($("#allDiffSlider").hasClass("height-transition-hidden"))
												        $("#allDiffSlider").slideDownTransition();
												});
										</script>
										</a><br />
								</xsl:for-each>
								<br />
								Tests: <br />
								<xsl:for-each select="//test[.//stateDifference//rootElementDifference/@screenId = $window/@screenId]">
										<a>
										<xsl:attribute name="href">#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" /></xsl:attribute>
										<xsl:attribute name="id">screenTestLink<xsl:value-of select="@screenId"/></xsl:attribute>
										<xsl:value-of select="@name"/>
										<script type="text/javascript">
												$("#screenTestLink<xsl:value-of select="@screenId"/>").click(function () {            
												    if ($("#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />").hasClass("height-transition-hidden"))
												        $("#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />").slideDownTransition();
												});
										</script>
										</a><br />
								</xsl:for-each>
								</td>
							</tr>
						</xsl:for-each>
					</table>
					</div></div>
					
					<h1><a href="javascript:;" id="differencePerComponent">Differences per Component</a></h1>
					<script type="text/javascript">
							$("#differencePerComponent").click(function () {            
							    if ($("#compDiffSlider").hasClass("height-transition-hidden"))
							        $("#compDiffSlider").slideDownTransition();
							    else
							        $("#compDiffSlider").slideUpTransition();
							});
					</script>
					<div id="compDiffSlider" class="height-transition height-transition-hidden"><div>
					<table class="summaryTable">
						<xsl:for-each select="$elementDifference">
							<xsl:sort select="@retestId" />
							<xsl:variable name="elementDifferenceEl" select="."/>
							<tr>
							<xsl:attribute name="class">component<xsl:value-of select="@retestId"/></xsl:attribute>
								<td>
									<input type="checkbox">
										<xsl:attribute name="name">component<xsl:value-of select="@retestId"/>Check</xsl:attribute>
										<xsl:attribute name="id">component<xsl:value-of select="@retestId"/>Check</xsl:attribute>
										<xsl:attribute name="onclick">markComponent('component<xsl:value-of select="@retestId"/>');</xsl:attribute>
										<xsl:call-template name="stripLast">
											<xsl:with-param name="pText" select=".//attribute[@key='type']/text()"/>
										</xsl:call-template>
										<xsl:if test=".//attribute[@key='text']">
                                           "<xsl:value-of select="normalize-space(.//attribute[@key='text']/text())"/>"
                                        </xsl:if>
										<xsl:if test=".//attribute[@key='context']">
                                           <xsl:value-of select="normalize-space(.//attribute[@key='context']/text())"/>
                                        </xsl:if>
									</input>
								</td>
								<td>
									<xsl:choose>
                                        <xsl:when test="descendant::expectedScreenshot">
											<img class="screenShot">
												<xsl:attribute name="src">screenshot/<xsl:value-of select="descendant::expectedScreenshot/persistenceId/text()" />.png</xsl:attribute>
											</img>
										</xsl:when>
										<xsl:when test="descendant::attributeDifference">
                                            <xsl:for-each select="./attributeDifference">
                                                <xsl:value-of select="@key"/>=<xsl:value-of select="text()"/><br />
                                            </xsl:for-each>
										</xsl:when>
										<xsl:when test="descendant::expected and not(descendant::actual)">
											component missing!
										</xsl:when>
										<xsl:when test="descendant::actual and not(descendant::expected)">
											component new!
										</xsl:when>
									</xsl:choose>
								</td>
								<td>
								Screens: <br />
								<xsl:for-each select="$rootElementDifference[(.//descendant::elementDifference/@retestId = $elementDifferenceEl/@retestId)]">
										<a>
										<xsl:attribute name="href">#screen<xsl:value-of select="@screenId" /></xsl:attribute>
										<xsl:attribute name="id">compScreenLink<xsl:value-of select="$elementDifferenceEl/@retestId"/></xsl:attribute>
										<xsl:value-of select="@windowDescription"/>
										<script type="text/javascript">
												$("#compScreenLink<xsl:value-of select="$elementDifferenceEl/@retestId"/>").click(function () {            
												    if ($("#screenDiffSlider").hasClass("height-transition-hidden"))
												        $("#screenDiffSlider").slideDownTransition();
												});
										</script>
										</a><br />
								</xsl:for-each>
								<br />
								Tests: <br />
								<xsl:for-each select="//test[child::action/stateDifference/difference//elementDifference/@retestId = $elementDifferenceEl/@retestId]">
										<a>
										<xsl:attribute name="href">#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" /></xsl:attribute>
										<xsl:attribute name="id">compTestLink<xsl:value-of select="$elementDifferenceEl/@retestId"/></xsl:attribute>
										<xsl:value-of select="@name"/>
										<script type="text/javascript">
												$("#compTestLink<xsl:value-of select="$elementDifferenceEl/@retestId"/>").click(function () {            
												    if ($("#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />").hasClass("height-transition-hidden"))
												        $("#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />").slideDownTransition();
												});
										</script>
										</a><br />
								</xsl:for-each>
								</td>
							</tr>
						</xsl:for-each>
					</table>
					</div></div>

                    <xsl:if test="$attributeDifference">
                    <h1><a href="javascript:;" id="differencePerAttributeDifference">Differences per Attribute</a></h1>
                    <script type="text/javascript">
                            $("#differencePerAttributeDifference").click(function () {            
                                if ($("#attrDiffSlider").hasClass("height-transition-hidden"))
                                    $("#attrDiffSlider").slideDownTransition();
                                else
                                    $("#attrDiffSlider").slideUpTransition();
                            });
                    </script>
                    <div id="attrDiffSlider" class="height-transition height-transition-hidden"><div>
                    <table class="summaryTable">
                            <tr>
                                <th>Attribute</th>
                                <th>Expected</th>
                                <th>Actual</th>
                                <th>Affects</th>
                            </tr>
                        <xsl:for-each select="$attributeDifference">
                            <xsl:sort select="@key" />
                            <xsl:variable name="attributeDifferenceEl" select="."/>
                            <tr>
                            <xsl:attribute name="class">attributeDifference<xsl:value-of select="@attributeDifferenceId"/></xsl:attribute>
                                <td>
                                    <input type="checkbox">
                                        <xsl:attribute name="name">attributeDifference<xsl:value-of select="@attributeDifferenceId"/>Check</xsl:attribute>
                                        <xsl:attribute name="id">attributeDifference<xsl:value-of select="@attributeDifferenceId"/>Check</xsl:attribute>
                                        <xsl:attribute name="onclick">markAttributeDifference('attributeDifference<xsl:value-of select="@attributeDifferenceId"/>');</xsl:attribute>
                                    <xsl:value-of select="$attributeDifferenceEl/@key" />                                    
                                    </input>
                                </td>
                                <td>
                                    <xsl:value-of select="$attributeDifferenceEl/expected/text()" />
                                </td>
                                <td>
                                    <xsl:value-of select="$attributeDifferenceEl/actual/text()" />
                                </td>
                                <td>
                                Components: <br />
                                <xsl:for-each select="$elementDifference[descendant::attributeDifference/@attributeDifferenceId = $attributeDifferenceEl/@attributeDifferenceId]">
                                    <xsl:choose>
                                        <xsl:when test="descendant::expectedScreenshot">
	                                        <img class="screenShot">
	                                            <xsl:attribute name="src">screenshot/<xsl:value-of select="descendant::expectedScreenshot/persistenceId/text()" />.png</xsl:attribute>
	                                        </img>
	                                        <br />
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:value-of select="./attributeDifference[@key='text']"/> (<xsl:value-of select="./attributeDifference[@key='type']"/>)<br />
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:for-each>
                                <br />
                                Tests: <br />
                                <xsl:for-each select="//test[descendant::attributeDifference/@attributeDifferenceId = $attributeDifferenceEl/@attributeDifferenceId]">
                                        <a>
                                        <xsl:attribute name="href">#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" /></xsl:attribute>
                                        <xsl:value-of select="@name"/>
                                        </a><br />
                                </xsl:for-each>
                                </td>
                            </tr>
                        </xsl:for-each>
                    </table>
                    </div></div>
					</xsl:if>
					
				</section>
				</xsl:if>
				
				<xsl:if test="//error or //targetNotFound">
				<section id="errorSummarySection">
					<h1><a href="javascript:;" id="errorSummary">Error Summary</a></h1>
					<script type="text/javascript">
							$("#errorSummary").click(function () {            
							    if ($("#errorSlider").hasClass("height-transition-hidden"))
							        $("#errorSlider").slideDownTransition();
							    else
							        $("#errorSlider").slideUpTransition();
							});
					</script>
					<div id="errorSlider" class="height-transition height-transition-hidden"><div>
					<table class="summaryTable errorTable">
						<xsl:for-each select="//action/error[not(./@errorId = ./preceding::action/error/@errorId)]">
							<xsl:sort select="@errorId" />
							<tr>
							<xsl:attribute name="class">errorRow error<xsl:value-of select="@errorId"/></xsl:attribute>
								<xsl:call-template name="errorTmpl">
									<xsl:with-param name="error" select="." />
								</xsl:call-template>
							</tr>
						</xsl:for-each>
						<xsl:for-each select="//targetNotFound">
							<tr class="errorRow">
								<xsl:call-template name="targetNotFoundTmpl">
									<xsl:with-param name="targetNotFound" select="." />
								</xsl:call-template>
							</tr>
						</xsl:for-each>
					</table>
					</div></div>
				</section>
				</xsl:if>

				<section id="testsSection">
				<a name="tests" />
				<xsl:for-each select="//suite">
				<h1>
					<a href="javascript:;" class="suiteHeader">
						<xsl:attribute name="id">suite<xsl:value-of select="@suiteNr" />Header</xsl:attribute>
						<xsl:value-of select="@name" /> 
						<span class="suiteHeaderInfo">
						  Tests: <span><xsl:value-of select="count(test)" /></span><br />
						  Actions: <span><xsl:value-of select="count(test/action)" /></span><br />
						  Duration: <span><xsl:value-of select="suiteDuration" /> ms</span><br />
						</span>
					</a>
				</h1>
				
				<xsl:for-each select="test">
					<h2>
						<a href="javascript:;">
							<xsl:attribute name="class">
								<xsl:if test="*/error or */targetNotFound">errorRow</xsl:if>
								<xsl:for-each select="*/stateDifference[not(./@differenceId = ./preceding::test/stateDifference/@differenceId)]">
									difference<xsl:value-of select="@differenceId"/>
								</xsl:for-each>
								testHeader
							</xsl:attribute>
							<xsl:attribute name="id">suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />Header</xsl:attribute>
							<xsl:value-of select="@name" /> 
							<span class="testHeaderInfo">
							   Actions: <span><xsl:value-of select="count(action)" /></span><br />
                               Duration: <span><xsl:value-of select="testDuration" /> ms</span><br />
							</span>
						</a>
					</h2>
					<script type="text/javascript">
							$("#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />Header").click(function () {            
							    if ($("#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />").hasClass("height-transition-hidden"))
							        $("#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />").slideDownTransition();
							    else
							        $("#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />").slideUpTransition();
							});
					</script>
					<div class="height-transition height-transition-hidden">
					<xsl:attribute name="id">suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" /></xsl:attribute><div>
					<table class="summaryTable">
					<xsl:attribute name="id">suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />Table</xsl:attribute>
						<tr>
							<th>Action</th>
							<th colspan="2">Result</th>
						</tr>
						<xsl:for-each select="action">
							<tr>
								<xsl:attribute name="class">
								    <xsl:if test="stateDifference">difference<xsl:value-of select="stateDifference/@differenceId" /></xsl:if>
								    <xsl:if test="error or targetNotFound">errorRow</xsl:if>
								</xsl:attribute>
								<!-- TODO RET-665: Here we want possibility to approve and change color with js. -->
							    <xsl:if test="@executionType">
									<xsl:attribute name="style">
										background-color: #FFFE89;
									</xsl:attribute>
							    </xsl:if>
								<xsl:attribute name="id">suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" /></xsl:attribute>
								<td>
									<xsl:if test="stateDifference">
										<xsl:variable name="rowspan">
											<xsl:choose>
												<xsl:when test="stateDifference/expected">3</xsl:when>
												<xsl:otherwise>2</xsl:otherwise>
											</xsl:choose>
										</xsl:variable>
										<xsl:attribute name="rowspan"><xsl:value-of select="$rowspan + count(descendant::elementDifference[not(descendant::elementDifference)])"/></xsl:attribute>
									</xsl:if>
									<xsl:if test="state">
										<xsl:attribute name="rowspan">2</xsl:attribute>
									</xsl:if>
									<xsl:value-of select="@description" />
									<xsl:if test="targetcomponent/screenshot">
										<img class="screenShot">
											<xsl:attribute name="src">screenshot/<xsl:value-of select="targetcomponent/screenshot/persistenceId/text()" />.png</xsl:attribute>
										</img>
									</xsl:if>
								</td>
								<xsl:if test="state">
									<td class="stateCol" colspan="2">
									    duration = <xsl:value-of select="@duration" /> ms
									</td>
									<script type="text/javascript">
                                            $("#suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />").click(function () {            
                                                if ($(".suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details").hasClass("height-transition-hidden")) {
                                                    $(".suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details").parent().removeClass("height-hidden");
                                                    $(".suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details").slideDownTransition();
                                                } else {
                                                    setTimeout(function() {$(".test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details").parent().addClass("height-hidden");}, 501);
                                                    $(".suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details").slideUpTransition();
                                                }
                                            });
                                    </script>
								</xsl:if>
								<xsl:if test="error">
									<xsl:call-template name="errorTmpl">
										<xsl:with-param name="error" select="error" />
									</xsl:call-template>
								</xsl:if>
								<xsl:if test="targetNotFound">
									<xsl:call-template name="targetNotFoundTmpl">
										<xsl:with-param name="targetNotFound" select="targetNotFound" />
									</xsl:call-template>
								</xsl:if>
								<xsl:if test="stateDifference">
								    <td class="stateExpectedCol">
                                        Expected
                                    </td>
                                    <td class="actualStateCol">
                                        Actual
	                                    <script type="text/javascript">
	                                            $("#suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />").click(function () {            
	                                                if ($(".suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details").hasClass("height-transition-hidden")) {
	                                                    $(".suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details").parent().removeClass("height-hidden");
	                                                    $(".suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details").slideDownTransition();
	                                                } else {
	                                                    setTimeout(function() {$(".test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details").parent().addClass("height-hidden");}, 501);
	                                                    $(".suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details").slideUpTransition();
	                                                }
	                                            });
	                                    </script>
                                    </td>
                                </xsl:if>
                                </tr>
                                <xsl:if test="state">
                                	<tr>
                                		<xsl:attribute name="class">state<xsl:value-of select="@id"/></xsl:attribute>
                                		<td class="stateCol height-hidden" colspan="2">
										    <div>
										    	<xsl:attribute name="class">suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details height-transition-hidden height-transition
										    	</xsl:attribute>
												<xsl:value-of select="state/window/@title" />
												<br />
												<xsl:if test="state/window/screenshot">
													<a target="_blank">
		                                            <xsl:attribute name="href">screenshot/<xsl:value-of select="state/window/screenshot/persistenceId/text()" />.png</xsl:attribute>
													<img class="screenShot">
														<xsl:attribute name="src">screenshot/<xsl:value-of select="state/window/screenshot/persistenceId/text()" />.png</xsl:attribute>
													</img>
													</a>
													<br />
												</xsl:if>
											</div>
										</td>
                                	</tr>
                                </xsl:if>
								<xsl:if test="stateDifference">
									<xsl:if test="stateDifference/expected">
										<tr>
											<xsl:attribute name="class">difference<xsl:value-of select="stateDifference/@differenceId"/></xsl:attribute>
											<xsl:variable name="divClass">suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details height-transition-hidden height-transition
											</xsl:variable>
											<td class="stateExpectedCol height-hidden">
												<div class="{$divClass}">
													duration = <xsl:value-of select="stateDifference/expected/attributeDifference"/> ms
												</div>
											</td>
											<td class="actualStateCol height-hidden">
												<div class="{$divClass}">
													duration = <xsl:value-of select="stateDifference/actual/attributeDifference"/> ms
												</div>
											</td>
										</tr>
									</xsl:if>
								    <tr>
	                                	<xsl:attribute name="class">
	                                		<xsl:if test="stateDifference">difference<xsl:value-of select="stateDifference/@differenceId" /></xsl:if>
	                                		<xsl:if test="error or targetNotFound">errorRow</xsl:if>
	                                	</xsl:attribute>
										<td class="stateExpectedCol height-hidden">
										   <div> 
											   <xsl:attribute name="class">suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details height-transition-hidden height-transition</xsl:attribute>
												<xsl:if test="stateDifference[not(./@differenceId = ./preceding::stateDifference/@differenceId)]">
													<a class="hiddenAnchor">
													<xsl:attribute name="name">difference<xsl:value-of select="stateDifference/@differenceId"/></xsl:attribute>
													Difference<xsl:value-of select="stateDifference/@differenceId"/>
													</a>
												</xsl:if>
												<xsl:value-of select="stateDifference//rootElementDifference/@windowDescription" />
												<br />
												<xsl:if test="stateDifference//rootElementDifference/expectedScreenshot">
												    <a target="_blank">
		                                            <xsl:attribute name="href">screenshot/<xsl:value-of select="stateDifference//rootElementDifference/expectedScreenshot/persistenceId/text()" />.png</xsl:attribute>
													<img class="screenShot">
														<xsl:attribute name="src">screenshot/<xsl:value-of select="stateDifference//rootElementDifference/expectedScreenshot/persistenceId/text()" />.png</xsl:attribute>
													</img>
													</a>
													<br />
												</xsl:if>
		                                        <xsl:for-each select="stateDifference//rootElementDifference/attributeDifference">
		                                          <xsl:value-of select="./@key" /> = <xsl:value-of select="./expected/text()" /><br />
		                                        </xsl:for-each>
	                                        </div>
										</td>
										<td class="actualStateCol height-hidden">
										   <div>
										    	<xsl:attribute name="class">suite<xsl:value-of select="../../@suiteNr" />test<xsl:value-of select="../@testNr" />Action<xsl:value-of select="@actionNr" />Details height-transition-hidden height-transition</xsl:attribute>
												<xsl:value-of select="stateDifference//rootElementDifference/@windowDescription" />
												<br />
												<xsl:if test="stateDifference//rootElementDifference/actualScreenshot">
												    <a target="_blank">
		                                            <xsl:attribute name="href">screenshot/<xsl:value-of select="stateDifference//rootElementDifference/actualScreenshot/persistenceId/text()" />.png</xsl:attribute>
													<img class="screenShot">
														<xsl:attribute name="src">screenshot/<xsl:value-of select="stateDifference//rootElementDifference/actualScreenshot/persistenceId/text()" />.png</xsl:attribute>
													</img>
													</a>
													<br />
												</xsl:if>
		                                        <xsl:for-each select="stateDifference//rootElementDifference/attributeDifference">
		                                          <xsl:value-of select="./@key" /> = <xsl:value-of select="./actual/text()" /><br />
		                                        </xsl:for-each>
	                                        </div>
										</td>
									</tr>
								</xsl:if>
								<!-- TODO Same if as above -->
								<xsl:if test="stateDifference">
									<xsl:for-each select="descendant::elementDifference[not(descendant::elementDifference)]">
										<tr>
										<xsl:attribute name="class">component<xsl:value-of select="@retestId"/></xsl:attribute>
										<td class="stateExpectedCol height-hidden">
										    <div>
										    <xsl:attribute name="class">suite<xsl:value-of select="ancestor::suite/@suiteNr" />test<xsl:value-of select="ancestor::test/@testNr" />Action<xsl:value-of select="ancestor::action/@actionNr" />Details height-transition-hidden height-transition</xsl:attribute>
											<xsl:if test="descendant::expectedScreenshot">
												<img class="screenShot">
													<xsl:attribute name="src">screenshot/<xsl:value-of select="descendant::expectedScreenshot/persistenceId/text()" />.png</xsl:attribute>
												</img>
												<br />
											</xsl:if>
											<xsl:choose>
												<xsl:when test="not(descendant::expected)">
												     missing
												</xsl:when>
												<xsl:when test="not(descendant::actual)">
													<xsl:call-template name="stripLast">
														<xsl:with-param name="pText" select="descendant::expected//attribute[@key='type']/text()"/>
													</xsl:call-template>
													<xsl:if test="./attributeDifference[@key='text']">
			                                           "<xsl:value-of select="normalize-space(descendant::expected//attribute[@key='text']/text())"/>"
			                                        </xsl:if>
			                                        <xsl:if test=".//attribute[@key='text']">
			                                        	"<xsl:value-of select="normalize-space(.//attribute[@key='text']/text())"/>"
			                                        </xsl:if>
												</xsl:when>
												<xsl:otherwise>
													<xsl:for-each select="descendant::expected">
											  		  <xsl:value-of select="../@key" /> = <xsl:value-of select="./text()" /><br />
		                                    		</xsl:for-each>
												</xsl:otherwise>
											</xsl:choose>
										    </div>
										</td>
										<td class="actualStateCol height-hidden">
										    <div>
										    <xsl:attribute name="class">suite<xsl:value-of select="ancestor::suite/@suiteNr" />test<xsl:value-of select="ancestor::test/@testNr" />Action<xsl:value-of select="ancestor::action/@actionNr" />Details height-transition-hidden height-transition</xsl:attribute>
											<xsl:if test="descendant::actualScreenshot">
												<img class="screenShot">
													<xsl:attribute name="src">screenshot/<xsl:value-of select="descendant::actualScreenshot/persistenceId/text()" />.png</xsl:attribute>
												</img>
												<br />
											</xsl:if>
											<xsl:choose>
												<xsl:when test="not(descendant::actual)">
												     missing
												</xsl:when>
												<xsl:when test="not(descendant::expected)">
													<xsl:call-template name="stripLast">
														<xsl:with-param name="pText" select="descendant::expected//attribute[@key='path']/text()"/>
													</xsl:call-template>
													<xsl:if test=".//attributeDifference[@key='text']">
			                                           "<xsl:value-of select="normalize-space(descendant::expected//attribute[@key='text']/text())"/>"
			                                        </xsl:if>
			                                        <xsl:if test=".//attribute[@key='text']">
			                                        	"<xsl:value-of select="normalize-space(.//attribute[@key='text']/text())"/>"
			                                        </xsl:if>
												</xsl:when>
												<xsl:otherwise>
		                                            <xsl:for-each select="descendant::actual">
		                                              <xsl:value-of select="../@key" /> = <xsl:value-of select="./text()" /><br />
		                                            </xsl:for-each>
												</xsl:otherwise>
											</xsl:choose>
                                            </div>
										</td>
										</tr>
									</xsl:for-each>
								</xsl:if>
						</xsl:for-each>
						
						<tr>
						<td colspan="3">
							<a href="javascript:;"> 
							<div class="closeBtn">
							<xsl:attribute name="id">closeTest<xsl:value-of select="@testNr" /></xsl:attribute>
							close <xsl:value-of select="@name" />
							<div>
							<script type="text/javascript">
									$("#closeTest<xsl:value-of select="@testNr" />").click(function () {            
									    if ($("#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />").hasClass("height-transition-hidden"))
									        $("#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />").slideDownTransition();
									    else
									        $("#suite<xsl:value-of select="../@suiteNr" />test<xsl:value-of select="@testNr" />").slideUpTransition();
									});
							</script>
						</div>
						</div>
						</a>
						</td>
						</tr>
					</table>
					</div></div>
				</xsl:for-each>
				</xsl:for-each>
				</section>
				</div>

				<footer>
					<div class="row">
						<div class="twelve columns">
							<ul class="copyright">
								<li>Copyright &#169;
									2018 ReTest GmbH
								</li>
								<li id="impressumslink">
									<a href="https://retest.de/impressum.html">Impressum</a>
								</li>
							</ul>
						</div>
						<div id="go-top" style="display: block;">
							<a title="Back to Top" href="#">Go To Top</a>
						</div>
						<div id="feedback" style="display: block;">
							<a title="Give Feedback" href="http://feedback.retest.de/">Feedback</a>
						</div>
					</div>
				</footer>
			</body>
		</html>
	</xsl:template>
	
	<xsl:template name="errorTmpl">
		<xsl:param name="error" />
		<td class="errorCol" colspan="2">
			Got following error:
			<xsl:value-of select="$error" />
		</td>
	</xsl:template>
	
	<xsl:template name="targetNotFoundTmpl">
		<xsl:param name="targetNotFound" />
		<td class="errorCol">
			<b>Action-Target not found!</b>
			<a target="_blank">
				<xsl:attribute name="href">screenshot/<xsl:value-of select="$targetNotFound//windowsScreenshots[1]/persistenceId/text()" />.png</xsl:attribute>
				<img class="screenShot">
	                <xsl:attribute name="src">screenshot/<xsl:value-of select="$targetNotFound//windowsScreenshots[1]/persistenceId/text()" />.png</xsl:attribute>
	            </img>
	        </a>
            <br /><xsl:value-of select="substring-before($targetNotFound/message,'Best with')"/>
			<img class="screenShot">
                <xsl:attribute name="src">screenshot/<xsl:value-of select="$targetNotFound//screenshot[1]/persistenceId/text()" />.png</xsl:attribute>
            </img>
		</td>
		<td class="errorCol">
			<a target="_blank">
				<xsl:attribute name="href">screenshot/<xsl:value-of select="$targetNotFound/windowsScreenshots/persistenceId/text()" />.png</xsl:attribute>
				<img class="screenShot">
	                <xsl:attribute name="src">screenshot/<xsl:value-of select="$targetNotFound/windowsScreenshots/persistenceId/text()" />.png</xsl:attribute>
	            </img>
	        </a>
            <br />Best with <xsl:value-of select="substring-after($targetNotFound/message,'Best with')"/>
            <xsl:if test="$targetNotFound/bestMatch/screenshot">
	            <img class="screenShot">
	                <xsl:attribute name="src">screenshot/<xsl:value-of select="$targetNotFound/bestMatch/screenshot/persistenceId/text()" />.png</xsl:attribute>
	            </img>
            </xsl:if>
		</td>
	</xsl:template>
	
	<xsl:template name="stripLast">
	  <xsl:param name="pText"/>
	  <xsl:param name="pDelim" select="'.'"/>
	
	   <xsl:choose>
	   <xsl:when test="contains($pText, $pDelim)">
	     <xsl:call-template name="stripLast">
	       <xsl:with-param name="pText" select="substring-after($pText, $pDelim)"/>
	       <xsl:with-param name="pDelim" select="$pDelim"/>
	     </xsl:call-template>
	   </xsl:when>
	     <xsl:otherwise>
		   <xsl:value-of select="$pText"/>
		 </xsl:otherwise>
	   </xsl:choose> 
	 </xsl:template>
</xsl:stylesheet>
